﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Forms;
using System.Threading;
using System.Threading.Tasks;
using System.Diagnostics;

namespace HladanieHesiel
{
    public partial class Form1 : Form
    {
        MD5[] md5;
        SHA_2[] sha2;
        Task[] ulohy;
        Thread[] vlakna;
        Excel excel;
        Stopwatch sw;
        int pocetPouzivatelskychKont;
        int volbaFunkcie;
        bool otvorenySubor;

        public Form1()
        {
            InitializeComponent();
            excel = new Excel();
            sw = new Stopwatch();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            cmbbTypHesla.SelectedItem = cmbbTypHesla.Items[0];
            cmbbDlzkaHesla.SelectedItem = cmbbDlzkaHesla.Items[0];
            cmbbFunkcia.SelectedItem = cmbbFunkcia.Items[0];
            cmbbVstupFunkcie.SelectedItem = cmbbVstupFunkcie.Items[0];

            dataGridView1.Columns.Add("Meno", "Meno");
            dataGridView1.Columns.Add("Hash", "Hash");
            dataGridView1.Columns.Add("Heslo", "Heslo");
        }

        /// <summary>
        /// Zabezpečí kontrolu vstupov od používateľa, otvorenie súboru excel a vytvorenie inštancií triedy MD5 alebo SHA_2.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void otvoritToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (cmbbTypHesla.SelectedItem == cmbbTypHesla.Items[0])
            {
                if ((cmbbDlzkaHesla.SelectedItem == cmbbDlzkaHesla.Items[0]) || (cmbbDlzkaHesla.SelectedItem == cmbbDlzkaHesla.Items[1]))
                {
                    if ((cmbbFunkcia.SelectedItem == cmbbFunkcia.Items[0]) || (cmbbFunkcia.SelectedItem == cmbbFunkcia.Items[1]))
                    {
                        if (cmbbVstupFunkcie.SelectedItem == cmbbVstupFunkcie.Items[0])
                        {
                            openFileDialog1.FileName = String.Empty;
                            openFileDialog1.Filter = "Súbor programu Excel 2007 (*.xlsx)|*xlsx|Súbor programu Excel 2003 (*.xls)|*xls;";
                            if (openFileDialog1.ShowDialog() == DialogResult.OK)
                            {
                                try
                                {
                                    excel.OtvoritSubor(openFileDialog1.FileName);

                                    pocetPouzivatelskychKont = excel.Tabulka.Rows.Count;
                                    RozdeleniePrace.RozdelPracu(pocetPouzivatelskychKont);

                                    vlakna = new Thread[RozdeleniePrace.PocetPracovnychJednotiek];
                                    ulohy = new Task[RozdeleniePrace.CelkoveMnozstvoPrace];

                                    if (cmbbFunkcia.SelectedItem == cmbbFunkcia.Items[0])
                                    {
                                        volbaFunkcie = 0;
                                        if (md5 != null)
                                        {
                                            foreach (MD5 instancia in md5)
                                            {
                                                if (instancia != null)
                                                {
                                                    instancia.Dispose();    
                                                }
                                            }
                                        }
                                        //Naplnenie poľa objektov triedy MD5
                                        md5 = new MD5[pocetPouzivatelskychKont];
                                        for (int i = 0; i < md5.Length; i++)
                                        {
                                            md5[i] = new MD5(excel.Tabulka.Rows[i][0].ToString(), excel.Tabulka.Rows[i][1].ToString());
                                        }
                                    }
                                    else if (cmbbFunkcia.SelectedItem == cmbbFunkcia.Items[1])
                                    {
                                        volbaFunkcie = 1;
                                        if (sha2 != null)
                                        {
                                            foreach (SHA_2 instancia in sha2)
                                            {
                                                if (instancia != null)
                                                {
                                                    instancia.Dispose();
                                                }
                                            }
                                        }
                                        //Naplnenie poľa objektov triedy SHA_2
                                        sha2 = new SHA_2[pocetPouzivatelskychKont];
                                        for (int i = 0; i < sha2.Length; i++)
                                        {
                                            sha2[i] = new SHA_2(excel.Tabulka.Rows[i][0].ToString(), excel.Tabulka.Rows[i][1].ToString());
                                        }
                                    }
                                    
                                    otvorenySubor = true;
                                    ZobrazUdaje();

                                    cmbbTypHesla.Enabled = false;
                                    cmbbDlzkaHesla.Enabled = false;
                                    cmbbFunkcia.Enabled = false;
                                    cmbbVstupFunkcie.Enabled = false;
                                    
                                    btnSekvencnyAlg.Enabled = true;
                                    btnVlakna.Enabled = true;
                                    btnUlohy.Enabled = true;
                                    btnParalelnyFor.Enabled = true;
                                    btnParalelnyForEach.Enabled = true;
                                    
                                    otvoritToolStripMenuItem.Enabled = false;
                                    ulozitToolStripMenuItem.Enabled = true;
                                }
                                catch (Exception ex)
                                {
                                    excel.UkonciSpojenie();
                                    otvorenySubor = false;
                                    MessageBox.Show(ex.Message, "Hľadanie hesiel", MessageBoxButtons.OK, MessageBoxIcon.Error);
                                }
                            }
                        }
                        else
                        {
                            MessageBox.Show("Zvolený druh vstupu pre hašovaciu funkciu nie je implementovaný!", "Hľadanie hesiel", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                    }
                    else
                    {
                        MessageBox.Show("Zvolená hašovacia funkcia nie je implementovaná!", "Hľadanie hesiel", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
                else
                {
                    MessageBox.Show("Zvolená dĺžka hesla nie je implementovaná!", "Hľadanie hesiel", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            else
            {
                MessageBox.Show("Zvolený typ hesla nie je implementovaný!", "Hladanie hesiel", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ulozitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (otvorenySubor && !backgroundWorker1.IsBusy)
            {
                List<string> zoznam = new List<string>(pocetPouzivatelskychKont);

                if (volbaFunkcie == 0)
                {
                    foreach (var prvok in md5)
                    {
                        zoznam.Add(prvok.Heslo);
                    }
                }
                else if (volbaFunkcie == 1)
                {
                    foreach (var prvok in sha2)
                    {
                        zoznam.Add(prvok.Heslo);
                    }
                }

                try
                {
                    excel.UlozitSubor(zoznam);
                    MessageBox.Show("Nájdené heslá boli úspešne uložené do vstupného súboru.", "Hľadanie hesiel", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                catch (Exception)
                {
                    MessageBox.Show("Súbor sa nepodarilo uložiť!", "Hľadanie hesiel", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }

                ulozitToolStripMenuItem.Enabled = false;
                otvoritToolStripMenuItem.Enabled = true;

                cmbbTypHesla.Enabled = true;
                cmbbDlzkaHesla.Enabled = true;
                cmbbFunkcia.Enabled = true;
                cmbbVstupFunkcie.Enabled = true;

                btnSekvencnyAlg.Enabled = false;
                btnVlakna.Enabled = false;
                btnUlohy.Enabled = false;
                btnParalelnyFor.Enabled = false;
                btnParalelnyForEach.Enabled = false;

                lblSekvencnyAlg.Text = String.Empty;
                lblVlakna.Text = String.Empty;
                lblUlohy.Text = String.Empty;
                lblParalelnyFor.Text = String.Empty;
                lblParalelnyForEach.Text = String.Empty;
                lblStatus.Text = String.Empty;
                progressBar1.Value = 0;
            }
            else
            {
                MessageBox.Show("Súbor sa nepodarilo uložiť!", "Hľadanie hesiel", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ZobrazUdaje()
        {
            dataGridView1.Rows.Clear();

            if (volbaFunkcie == 0)
            {
                for (int i = 0; i < pocetPouzivatelskychKont; i++)
                {
                    dataGridView1.Rows.Add();
                    dataGridView1[0, i].Value = md5[i].Meno;
                    dataGridView1[1, i].Value = md5[i].Hash;
                    dataGridView1[2, i].Value = md5[i].Heslo;
                }
            }
            else if (volbaFunkcie == 1)
            {
                for (int i = 0; i < pocetPouzivatelskychKont; i++)
                {
                    dataGridView1.Rows.Add();
                    dataGridView1[0, i].Value = sha2[i].Meno;
                    dataGridView1[1, i].Value = sha2[i].Hash;
                    dataGridView1[2, i].Value = sha2[i].Heslo;
                }
            }
        }

        private void ZmazHesla()
        {
            if (volbaFunkcie == 0)
            {
                for (int i = 0; i < pocetPouzivatelskychKont; i++)
                {
                    md5[i].Heslo = String.Empty;
                }
            }
            else if (volbaFunkcie == 1)
            {
                for (int i = 0; i < pocetPouzivatelskychKont; i++)
                {
                    sha2[i].Heslo = String.Empty;
                }
            }
        }

        private bool NastavDlzkuHesla()
        {
            if ((cmbbDlzkaHesla.SelectedItem == cmbbDlzkaHesla.Items[0]) || (cmbbDlzkaHesla.SelectedItem == cmbbDlzkaHesla.Items[1]))
            {
                if (volbaFunkcie == 0)
                {
                    for (int i = 0; i < md5.Length; i++)
                    {
                        md5[i].DlzkaHesla = cmbbDlzkaHesla.SelectedItem.ToString();
                    }
                }
                else if (volbaFunkcie == 1)
                {
                    for (int i = 0; i < sha2.Length; i++)
                    {
                        sha2[i].DlzkaHesla = cmbbDlzkaHesla.SelectedItem.ToString();
                    }
                }
                return true;
            }
            else
            {
                MessageBox.Show("Zvolená dĺžka hesla nie je implementovaná!", "Hľadanie hesiel", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
        }
        
        /// <summary>
        /// Metóda pripraví komponent backgroundWorker1 na použitie.
        /// </summary>
        /// <param name="volbaAlgoritmu">
        /// volbaAlgoritmu = 0   - na nájdenie hesiel sa použije sekvenčný algoritmus;
        /// volbaAlgoritmu = 1   - na nájdenie hesiel sa použije paralelný algoritmus (s použitím triedy Thread);
        /// volbaAlgoritmu = 2   - na nájdenie hesiel sa použije paralelný algoritmus (s použitím triedy Task);
        /// volbaAlgoritmu = 3   - na nájdenie hesiel sa použije paralelný algoritmus (s použitím statickej metódy Paralel.For);
        /// volbaAlgoritmu = 4   - na nájdenie hesiel sa použije paralelný algoritmus (s použitím statickej metódy Paralel.ForEach);
        /// </param>
        private void InicializujBackgroundWorker(int volbaAlgoritmu)
        {
            switch (volbaAlgoritmu)
            {
                case 0:
                    backgroundWorker1.DoWork -= backgroundWorker1SekvencnyAlg_DoWork;
                    backgroundWorker1.RunWorkerCompleted -= backgroundWorker1SekvencnyAlg_RunWorkerCompleted;
                    backgroundWorker1.DoWork -= backgroundWorker1Vlakna_DoWork;
                    backgroundWorker1.RunWorkerCompleted -= backgroundWorker1Vlakna_RunWorkerCompleted;
                    backgroundWorker1.DoWork -= backgroundWorker1Ulohy_DoWork;
                    backgroundWorker1.RunWorkerCompleted -= backgroundWorker1Ulohy_RunWorkerCompleted;
                    backgroundWorker1.DoWork -= backgroundWorker1ParalelnyFor_DoWork;
                    backgroundWorker1.RunWorkerCompleted -= backgroundWorker1ParalelnyFor_RunWorkerCompleted;
                    backgroundWorker1.DoWork -= backgroundWorker1ParalelnyForEach_DoWork;
                    backgroundWorker1.RunWorkerCompleted -= backgroundWorker1ParalelnyForEach_RunWorkerCompleted;
                    
                    backgroundWorker1.DoWork += new DoWorkEventHandler(backgroundWorker1SekvencnyAlg_DoWork);
                    backgroundWorker1.RunWorkerCompleted += new RunWorkerCompletedEventHandler(backgroundWorker1SekvencnyAlg_RunWorkerCompleted);
                    break;
                case 1:
                    backgroundWorker1.DoWork -= backgroundWorker1SekvencnyAlg_DoWork;
                    backgroundWorker1.RunWorkerCompleted -= backgroundWorker1SekvencnyAlg_RunWorkerCompleted;
                    backgroundWorker1.DoWork -= backgroundWorker1Vlakna_DoWork;
                    backgroundWorker1.RunWorkerCompleted -= backgroundWorker1Vlakna_RunWorkerCompleted;
                    backgroundWorker1.DoWork -= backgroundWorker1Ulohy_DoWork;
                    backgroundWorker1.RunWorkerCompleted -= backgroundWorker1Ulohy_RunWorkerCompleted;
                    backgroundWorker1.DoWork -= backgroundWorker1ParalelnyFor_DoWork;
                    backgroundWorker1.RunWorkerCompleted -= backgroundWorker1ParalelnyFor_RunWorkerCompleted;
                    backgroundWorker1.DoWork -= backgroundWorker1ParalelnyForEach_DoWork;
                    backgroundWorker1.RunWorkerCompleted -= backgroundWorker1ParalelnyForEach_RunWorkerCompleted;
                   
                    backgroundWorker1.DoWork += new DoWorkEventHandler(backgroundWorker1Vlakna_DoWork);
                    backgroundWorker1.RunWorkerCompleted += new RunWorkerCompletedEventHandler(backgroundWorker1Vlakna_RunWorkerCompleted);
                    break;
                case 2:
                    backgroundWorker1.DoWork -= backgroundWorker1SekvencnyAlg_DoWork;
                    backgroundWorker1.RunWorkerCompleted -= backgroundWorker1SekvencnyAlg_RunWorkerCompleted;
                    backgroundWorker1.DoWork -= backgroundWorker1Vlakna_DoWork;
                    backgroundWorker1.RunWorkerCompleted -= backgroundWorker1Vlakna_RunWorkerCompleted;
                    backgroundWorker1.DoWork -= backgroundWorker1Ulohy_DoWork;
                    backgroundWorker1.RunWorkerCompleted -= backgroundWorker1Ulohy_RunWorkerCompleted;
                    backgroundWorker1.DoWork -= backgroundWorker1ParalelnyFor_DoWork;
                    backgroundWorker1.RunWorkerCompleted -= backgroundWorker1ParalelnyFor_RunWorkerCompleted;
                    backgroundWorker1.DoWork -= backgroundWorker1ParalelnyForEach_DoWork;
                    backgroundWorker1.RunWorkerCompleted -= backgroundWorker1ParalelnyForEach_RunWorkerCompleted;
                     
                    backgroundWorker1.DoWork += new DoWorkEventHandler(backgroundWorker1Ulohy_DoWork);
                    backgroundWorker1.RunWorkerCompleted += new RunWorkerCompletedEventHandler(backgroundWorker1Ulohy_RunWorkerCompleted);
                    break;
                case 3:
                    backgroundWorker1.DoWork -= backgroundWorker1SekvencnyAlg_DoWork;
                    backgroundWorker1.RunWorkerCompleted -= backgroundWorker1SekvencnyAlg_RunWorkerCompleted;
                    backgroundWorker1.DoWork -= backgroundWorker1Vlakna_DoWork;
                    backgroundWorker1.RunWorkerCompleted -= backgroundWorker1Vlakna_RunWorkerCompleted;
                    backgroundWorker1.DoWork -= backgroundWorker1Ulohy_DoWork;
                    backgroundWorker1.RunWorkerCompleted -= backgroundWorker1Ulohy_RunWorkerCompleted;
                    backgroundWorker1.DoWork -= backgroundWorker1ParalelnyFor_DoWork;
                    backgroundWorker1.RunWorkerCompleted -= backgroundWorker1ParalelnyFor_RunWorkerCompleted;
                    backgroundWorker1.DoWork -= backgroundWorker1ParalelnyForEach_DoWork;
                    backgroundWorker1.RunWorkerCompleted -= backgroundWorker1ParalelnyForEach_RunWorkerCompleted;
                     
                    backgroundWorker1.DoWork += new DoWorkEventHandler(backgroundWorker1ParalelnyFor_DoWork);
                    backgroundWorker1.RunWorkerCompleted += new RunWorkerCompletedEventHandler(backgroundWorker1ParalelnyFor_RunWorkerCompleted);
                    break;
                case 4:
                    backgroundWorker1.DoWork -= backgroundWorker1SekvencnyAlg_DoWork;
                    backgroundWorker1.RunWorkerCompleted -= backgroundWorker1SekvencnyAlg_RunWorkerCompleted;
                    backgroundWorker1.DoWork -= backgroundWorker1Vlakna_DoWork;
                    backgroundWorker1.RunWorkerCompleted -= backgroundWorker1Vlakna_RunWorkerCompleted;
                    backgroundWorker1.DoWork -= backgroundWorker1Ulohy_DoWork;
                    backgroundWorker1.RunWorkerCompleted -= backgroundWorker1Ulohy_RunWorkerCompleted;
                    backgroundWorker1.DoWork -= backgroundWorker1ParalelnyFor_DoWork;
                    backgroundWorker1.RunWorkerCompleted -= backgroundWorker1ParalelnyFor_RunWorkerCompleted;
                    backgroundWorker1.DoWork -= backgroundWorker1ParalelnyForEach_DoWork;
                    backgroundWorker1.RunWorkerCompleted -= backgroundWorker1ParalelnyForEach_RunWorkerCompleted;
                     
                    backgroundWorker1.DoWork += new DoWorkEventHandler(backgroundWorker1ParalelnyForEach_DoWork);
                    backgroundWorker1.RunWorkerCompleted += new RunWorkerCompletedEventHandler(backgroundWorker1ParalelnyForEach_RunWorkerCompleted);
                    break;
                default:
                    break;
            }
        }

        private void backgroundWorker1_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            lblStatus.Text = e.ProgressPercentage.ToString() + " %";
            progressBar1.Value = e.ProgressPercentage;
        }


        #region Sekvenčný algoritmus
        private void btnSekvencnyAlg_Click(object sender, EventArgs e)
        {
            if (NastavDlzkuHesla())
            {
                ZmazHesla();
                ZobrazUdaje();
                if (!backgroundWorker1.IsBusy)
                {
                    InicializujBackgroundWorker(0);
                    // Štart asynchrónnej operácie...
                    backgroundWorker1.RunWorkerAsync();
                    lblSekvencnyAlg.Text = "Prebieha hľadanie hesiel sekvenčným algoritmom...";
                }
            }
        }
        
        private void backgroundWorker1SekvencnyAlg_DoWork(object sender, DoWorkEventArgs e)
        {
            sw.Start();
            backgroundWorker1.ReportProgress(0);
            if (volbaFunkcie == 0)
                for (int i = 0; i < md5.Length; i++)
                {
                    md5[i].NajdiHeslo();
                    backgroundWorker1.ReportProgress((int)((float)(i + 1) / (float)md5.Length * 100));
                }
            else if (volbaFunkcie == 1)
                for (int i = 0; i < sha2.Length; i++)
                {
                    sha2[i].NajdiHeslo();
                    backgroundWorker1.ReportProgress((int)((float)(i + 1) / (float)sha2.Length * 100));
                }
            sw.Stop();
            e.Result = sw.Elapsed.TotalSeconds.ToString("0.00");
            sw.Reset();
        }
        
        private void backgroundWorker1SekvencnyAlg_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Error != null)
                MessageBox.Show(e.Error.Message, "Hľadanie hesiel", MessageBoxButtons.OK, MessageBoxIcon.Error);
            else
            {
                lblSekvencnyAlg.Text = "Exekučný čas sekvenčného algoritmu: " + e.Result.ToString() + " s.";
                ZobrazUdaje();
            }
        }
        #endregion


        #region Vlákna
        private void btnVlakna_Click(object sender, EventArgs e)
        {
            if (NastavDlzkuHesla())
            {
                ZmazHesla();
                ZobrazUdaje();
                if (!backgroundWorker1.IsBusy)
                {
                    InicializujBackgroundWorker(1);
                    // Štart asynchrónnej operácie...
                    backgroundWorker1.RunWorkerAsync();
                    lblVlakna.Text = "Prebieha hľadanie hesiel paralelným algoritmom...";
                }
                ZobrazUdaje();
            }
        }

        private void backgroundWorker1Vlakna_DoWork(object sender, DoWorkEventArgs e)
        {
            sw.Start();

            int aktualnyProgres = 0;
            backgroundWorker1.ReportProgress(aktualnyProgres);
            if (volbaFunkcie == 0)
                for (int i = 0; i < vlakna.Length; i++)
                {
                    if (i < (vlakna.Length - 1))
                        vlakna[i] = new Thread((object indexVlakna) =>
                        {
                            int index = Convert.ToInt32(indexVlakna);
                            for (int j = index * RozdeleniePrace.MnozstvoPraceVPracovnejJednotke; j < ((index + 1) * RozdeleniePrace.MnozstvoPraceVPracovnejJednotke); j++)
                            {
                                md5[j].NajdiHeslo();
                                backgroundWorker1.ReportProgress((int)((float)Interlocked.Increment(ref aktualnyProgres) / (float)RozdeleniePrace.CelkoveMnozstvoPrace * 100));
                            }
                        });
                    else
                        vlakna[i] = new Thread((object indexVlakna) =>
                        {
                            int index = Convert.ToInt32(indexVlakna);
                            for (int j = index * RozdeleniePrace.MnozstvoPraceVPracovnejJednotke; j < RozdeleniePrace.CelkoveMnozstvoPrace; j++)
                            {
                                md5[j].NajdiHeslo();
                                backgroundWorker1.ReportProgress((int)((float)Interlocked.Increment(ref aktualnyProgres) / (float)RozdeleniePrace.CelkoveMnozstvoPrace * 100));
                            }
                        });
                }
            else if (volbaFunkcie == 1)
                for (int i = 0; i < vlakna.Length; i++)
                {
                    if (i < (vlakna.Length - 1))
                        vlakna[i] = new Thread((object indexVlakna) =>
                        {
                            int index = Convert.ToInt32(indexVlakna);
                            for (int j = index * RozdeleniePrace.MnozstvoPraceVPracovnejJednotke; j < ((index + 1) * RozdeleniePrace.MnozstvoPraceVPracovnejJednotke); j++)
                            {
                                sha2[j].NajdiHeslo();
                                backgroundWorker1.ReportProgress((int)((float)Interlocked.Increment(ref aktualnyProgres) / (float)RozdeleniePrace.CelkoveMnozstvoPrace * 100));
                            }
                        });
                    else
                        vlakna[i] = new Thread((object indexVlakna) =>
                        {
                            int index = Convert.ToInt32(indexVlakna);
                            for (int j = index * RozdeleniePrace.MnozstvoPraceVPracovnejJednotke; j < RozdeleniePrace.CelkoveMnozstvoPrace; j++)
                            {
                                sha2[j].NajdiHeslo();
                                backgroundWorker1.ReportProgress((int)((float)Interlocked.Increment(ref aktualnyProgres) / (float)RozdeleniePrace.CelkoveMnozstvoPrace * 100));
                            }
                        });
                }
            
            for (int i = 0; i < vlakna.Length; i++)
            {
                vlakna[i].Start(i);
            }
            for (int i = 0; i < vlakna.Length; i++)
            {
                vlakna[i].Join();
            }

            sw.Stop();
            e.Result = sw.Elapsed.TotalSeconds.ToString("0.00");
            sw.Reset();
        }

        private void backgroundWorker1Vlakna_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Error != null)
                MessageBox.Show(e.Error.Message, "Hľadanie hesiel", MessageBoxButtons.OK, MessageBoxIcon.Error);
            else
            {
                lblVlakna.Text = "Exekučný čas paralelného algoritmu: " + e.Result.ToString() + " s.";
                ZobrazUdaje();
            }
        }
        #endregion


        #region Úlohy
        private void btnUlohy_Click(object sender, EventArgs e)
        {
            if (NastavDlzkuHesla())
            {
                ZmazHesla();
                ZobrazUdaje();
                if (!backgroundWorker1.IsBusy)
                {
                    InicializujBackgroundWorker(2);
                    // Štart asynchrónnej operácie...
                    backgroundWorker1.RunWorkerAsync();
                    lblUlohy.Text = "Prebieha hľadanie hesiel paralelným algoritmom...";
                }
                ZobrazUdaje();
            }
        }

        private void backgroundWorker1Ulohy_DoWork(object sender, DoWorkEventArgs e)
        {
            sw.Start();

            if (volbaFunkcie == 0)
                for (int i = 0; i < ulohy.Length; i++)
                {
                    ulohy[i] = new Task(md5[i].NajdiHeslo);
                }
            else if (volbaFunkcie == 1)
                for (int i = 0; i < ulohy.Length; i++)
                {
                    ulohy[i] = new Task(sha2[i].NajdiHeslo);
                }
            
            for (int i = 0; i < ulohy.Length; i++)
            {
                ulohy[i].Start();
            }
            backgroundWorker1.ReportProgress(0);
            for (int i = 0; i < ulohy.Length; i++)
            {
                ulohy[i].Wait();
                backgroundWorker1.ReportProgress((int)((float)(i + 1) / (float)ulohy.Length * 100));
            }

            sw.Stop();
            e.Result = sw.Elapsed.TotalSeconds.ToString("0.00");
            sw.Reset();
        }

        private void backgroundWorker1Ulohy_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Error != null)
                MessageBox.Show(e.Error.Message, "Hľadanie hesiel", MessageBoxButtons.OK, MessageBoxIcon.Error);
            else
            {
                lblUlohy.Text = "Exekučný čas paralelného algoritmu: " + e.Result.ToString() + " s.";
                ZobrazUdaje();
            }
        }
        #endregion


        #region Paralelný cyklus For
        private void btnParalelnyFor_Click(object sender, EventArgs e)
        {
            if (NastavDlzkuHesla())
            {
                ZmazHesla();
                ZobrazUdaje();
                if (!backgroundWorker1.IsBusy)
                {
                    InicializujBackgroundWorker(3);
                    // Štart asynchrónnej operácie...
                    backgroundWorker1.RunWorkerAsync();
                    lblParalelnyFor.Text = "Prebieha hľadanie hesiel paralelným algoritmom...";
                }
                ZobrazUdaje();
            }
        }

        private void backgroundWorker1ParalelnyFor_DoWork(object sender, DoWorkEventArgs e)
        {
            sw.Start();
            int aktualnyProgres = 0;
            backgroundWorker1.ReportProgress(aktualnyProgres);
            if (volbaFunkcie == 0)
                Parallel.For(0, RozdeleniePrace.CelkoveMnozstvoPrace, i =>
                {
                    md5[i].NajdiHeslo();
                    backgroundWorker1.ReportProgress((int)((float)Interlocked.Increment(ref aktualnyProgres) / (float)RozdeleniePrace.CelkoveMnozstvoPrace * 100));
                });
            else if (volbaFunkcie == 1)
                Parallel.For(0, RozdeleniePrace.CelkoveMnozstvoPrace, i =>
                {
                    sha2[i].NajdiHeslo();
                    backgroundWorker1.ReportProgress((int)((float)Interlocked.Increment(ref aktualnyProgres) / (float)RozdeleniePrace.CelkoveMnozstvoPrace * 100));
                });
            sw.Stop();
            e.Result = sw.Elapsed.TotalSeconds.ToString("0.00");
            sw.Reset();
        }

        private void backgroundWorker1ParalelnyFor_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Error != null)
                MessageBox.Show(e.Error.Message, "Hľadanie hesiel", MessageBoxButtons.OK, MessageBoxIcon.Error);
            else
            {
                lblParalelnyFor.Text = "Exekučný čas paralelného algoritmu: " + e.Result.ToString() + " s.";
                ZobrazUdaje();
            }
        }
        #endregion


        #region Paralelný cyklus ForEach
        private void btnParalelnyForEach_Click(object sender, EventArgs e)
        {
            if (NastavDlzkuHesla())
            {
                ZmazHesla();
                ZobrazUdaje();
                if (!backgroundWorker1.IsBusy)
                {
                    InicializujBackgroundWorker(4);
                    // Štart asynchrónnej operácie...
                    backgroundWorker1.RunWorkerAsync();
                    lblParalelnyForEach.Text = "Prebieha hľadanie hesiel paralelným algoritmom...";
                }
                ZobrazUdaje();
            }
        }

        private void backgroundWorker1ParalelnyForEach_DoWork(object sender, DoWorkEventArgs e)
        {
            sw.Start();
            int aktualnyProgres = 0;
            backgroundWorker1.ReportProgress(aktualnyProgres);
            if (volbaFunkcie == 0)
                Parallel.ForEach(md5, instanciaMd5 =>
                {
                    instanciaMd5.NajdiHeslo();
                    backgroundWorker1.ReportProgress((int)((float)Interlocked.Increment(ref aktualnyProgres) / (float)RozdeleniePrace.CelkoveMnozstvoPrace * 100));
                });
            else if (volbaFunkcie == 1)
                Parallel.ForEach(sha2, instanciaSha2 =>
                {
                    instanciaSha2.NajdiHeslo();
                    backgroundWorker1.ReportProgress((int)((float)Interlocked.Increment(ref aktualnyProgres) / (float)RozdeleniePrace.CelkoveMnozstvoPrace * 100));
                });
            sw.Stop();
            e.Result = sw.Elapsed.TotalSeconds.ToString("0.00");
            sw.Reset();
        }

        private void backgroundWorker1ParalelnyForEach_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Error != null)
                MessageBox.Show(e.Error.Message, "Hľadanie hesiel", MessageBoxButtons.OK, MessageBoxIcon.Error);
            else
            {
                lblParalelnyForEach.Text = "Exekučný čas paralelného algoritmu: " + e.Result.ToString() + " s.";
                ZobrazUdaje();
            }
        }
        #endregion
    }
}
